@@ -18,6 +18,7 @@
 #include <mutex>
 #include <cmath>
 #include <ctime>
+#include <memory>
 #include <atomic>
 #include <bitset>
 #include <vector>
@@ -49,6 +50,10 @@ namespace anyks {
 	 * Alm Класс модуля языковой модели
 	 */
 	typedef class Alm {
+		private:
+			// Объявляем дружественные классы
+			friend class Alm1;
+			friend class Alm2;
 		private:
 			/**
 			 * UserToken Структура пользовательского токена
@@ -71,43 +76,6 @@ namespace anyks {
 				 */
 				UserToken() : idw(idw_t::NIDW), name("") {}
 			} utoken_t;
-			/**
-			 * Структура словаря языковой модели
-			 */
-			typedef struct Arpa: std::map <size_t, Arpa> {
-				/**
-				 * Регистры слова
-				 */
-				size_t uppers;
-				/**
-				 * Частота последовательности и
-				 * обратная частота последовательности
-				 */
-				double weight, backoff;
-				/**
-				 * Arpa Конструктор
-				 */
-				Arpa() : uppers(0), weight(log(0)), backoff(0.0) {}
-			} arpa_t;
-			/**
-			 * Структура слова для карты последовательности
-			 */
-			typedef struct Seq {
-				/**
-				 * Частота n-граммы и обратная частота
-				 */
-				double weight, backoff;
-				/**
-				 * Идентификатор слова, встречаемость n-граммы,
-				 * количество документов где встретилась n-грамма,
-				 * значение регистров слова
-				 */
-				size_t idw, oc, dc, ups;
-				/**
-				 * Seq Конструктор
-				 */
-				Seq() : weight(log(0)), backoff(0.0), idw(idw_t::NIDW), oc(0), dc(0), ups(0) {}
-			} seq_t;
 		public:
 			// Основные опции
 			enum class options_t : u_short {
@@ -130,6 +98,25 @@ namespace anyks {
 				 */
 				Perplexity() : oovs(0), words(0), sentences(0), zeroprobs(0), logprob(0.0), ppl(idw_t::NIDW), ppl1(idw_t::NIDW) {}
 			} ppl_t;
+			/**
+			 * Структура слова для карты последовательности
+			 */
+			typedef struct Seq {
+				/**
+				 * Частота n-граммы и обратная частота
+				 */
+				double weight, backoff;
+				/**
+				 * Идентификатор слова, встречаемость n-граммы,
+				 * количество документов где встретилась n-грамма,
+				 * значение регистров слова
+				 */
+				size_t idw, oc, dc, ups;
+				/**
+				 * Seq Конструктор
+				 */
+				Seq() : weight(log(0)), backoff(0.0), idw(idw_t::NIDW), oc(0), dc(0), ups(0) {}
+			} seq_t;
 			// Упрощаем тип функции для получения слова
 			typedef function <const word_t * (const size_t)> words_t;
 			// Упрощаем тип функции для добавления слова
@@ -166,8 +153,6 @@ namespace anyks {
 			// Список скриптов python
 			std::map <u_short, pair <string, size_t>> scripts;
 		private:
-			// Словарь языковой модели
-			mutable arpa_t arpa;
 			// Создаем тредпул
 			mutable tpool_t tpool;
 			// Мютекс блокировки потока
@@ -200,25 +185,51 @@ namespace anyks {
 			 */
 			const bool event(const size_t idw) const noexcept;
 			/**
-			 * word Метод извлечения слова по его идентификатору
-			 * @param idw идентификатор слова
-			 * @param ups регистры слова
-			 * @return    слово соответствующее идентификатору
+			 * isOption Метод проверки наличия опции
+			 * @param option опция для проверки
+			 * @return       результат проверки
 			 */
-			const word_t word(const size_t idw, const size_t ups = 0) const noexcept;
+			const bool isOption(const options_t option) const noexcept;
+			/**
+			 * exist Метод проверки существования последовательности
+			 * @param seq список слов последовательности
+			 * @return    результат проверки
+			 */
+			virtual const bool exist(const vector <size_t> & seq) const noexcept;
+		private:
+			/**
+			 * backoff Метод извлечения обратной частоты последовательности
+			 * @param seq последовательность для извлечения обратной частоты
+			 * @return    обратная частота последовательности
+			 */
+			virtual const double backoff(const vector <size_t> & seq) const noexcept;
+			/**
+			 * weight Метод извлечения веса последовательности
+			 * @param seq последовательность для извлечения веса
+			 * @return    вес последовательноси и n-грамма для которой она получена
+			 */
+			virtual const pair <u_short, double> weight(const vector <size_t> & seq) const noexcept;
+			/**
+			 * frequency Метод извлечения частоты n-граммы
+			 * @param seq список слов последовательности
+			 * @return    частота и обратная частота n-граммы
+			 */
+			virtual const pair <double, double> frequency(const vector <size_t> & seq) const noexcept;
 		private:
 			/**
 			 * set Метод установки последовательности в словарь
 			 * @param seq последовательность слов для установки
 			 */
-			void set(const vector <seq_t> & seq) const noexcept;
+			virtual void set(const vector <seq_t> & seq) const noexcept;
 			/**
 			 * set Метод установки последовательности в словарь
 			 * @param seq     список идентификаторов слов которые нужно добавить
+			 * @param uppers  список верхних регистров последнего слова последовательности
 			 * @param weight  вес n-граммы из файла arpa
 			 * @param backoff обратная частота документа из файла arpa
 			 */
-			void set(const vector <pair_t> & seq, const double weight, const double backoff) const noexcept;
+			virtual void set(const vector <size_t> & seq, const size_t uppers, const double weight, const double backoff) const noexcept;
+		private:
 			/**
 			 * clearShielding Функция удаления экранирования
 			 * @param word  слово в котором следует удалить экранирование
@@ -226,19 +237,6 @@ namespace anyks {
 			 * @param back  последний символ экранирования
 			 */
 			void clearShielding(const string & word, const string & front = "<", const string & back = ">") const noexcept;
-		private:
-			/**
-			 * weight Метод извлечения веса последовательности
-			 * @param seq последовательность для извлечения веса
-			 * @return    вес последовательноси и n-грамма для которой она получена
-			 */
-			const pair <u_short, double> weight(const vector <size_t> & seq) const noexcept;
-			/**
-			 * frequency Метод извлечения частоты n-граммы
-			 * @param seq список слов последовательности
-			 * @return    частота и обратная частота n-граммы
-			 */
-			const pair <double, double> frequency(const vector <size_t> & seq) const noexcept;
 			/**
 			 * pplCalculate Метод расчёта значения перплексии по весу
 			 * @param logprob вес последовательности n-грамм полученный из расчётов
@@ -247,27 +245,6 @@ namespace anyks {
 			 * @return        значение перплексии полученное при расчётах
 			 */
 			const pair <double, double> pplCalculate(const double logprob, const size_t words, const size_t oovs) const noexcept;
-		private:
-			/**
-			 * isOption Метод проверки наличия опции
-			 * @param option опция для проверки
-			 * @return       результат проверки
-			 */
-			const bool isOption(const options_t option) const noexcept;
-			/**
-			 * backoff Метод извлечения обратной частоты последовательности
-			 * @param seq последовательность для извлечения обратной частоты
-			 * @return    обратная частота последовательности
-			 */
-			const double backoff(const vector <size_t> & seq) const noexcept;
-		public:
-			/**
-			 * getIdw Метод генерирования идентификатора слова
-			 * @param  word  слово для генерации
-			 * @param  check нужно выполнить дополнительную проверку слова
-			 * @return       идентификатор слова
-			 */
-			const size_t getIdw(const wstring & word, const bool check = true) const noexcept;
 		public:
 			/**
 			 * perplexity Метод расчёта перплексии текста
@@ -286,7 +263,7 @@ namespace anyks {
 			 * @param  seq список последовательностей
 			 * @return     результат расчёта
 			 */
-			const ppl_t perplexity(const vector <size_t> & seq) const noexcept;
+			virtual const ppl_t perplexity(const vector <size_t> & seq) const noexcept;
 			/**
 			 * pplConcatenate Метод объединения перплексий
 			 * @param ppl1 первая перплексия
@@ -323,7 +300,7 @@ namespace anyks {
 			 * @param accurate режим точной проверки
 			 * @return         результат проверки
 			 */
-			const pair <bool, size_t> check(const vector <size_t> & seq, const bool accurate = false) const noexcept;
+			virtual const pair <bool, size_t> check(const vector <size_t> & seq, const bool accurate = false) const noexcept;
 		public:
 			/**
 			 * fixUppers Метод исправления регистров в тексте
@@ -337,22 +314,22 @@ namespace anyks {
 			 * @return     текст с исправленными регистрами слов
 			 */
 			const wstring fixUppers(const wstring & text) const noexcept;
-			/**
-			 * context Метод сборки текстового контекста из последовательности
-			 * @param seq  последовательность слов для сборки контекста
-			 * @param nwrd флаг разрешающий вывод системных токенов
-			 * @return     собранный текстовый контекст
-			 */
-			const wstring context(const vector <size_t> & seq, const bool nwrd = false) const noexcept;
 		public:
 			/**
-			 * clear Метод очистки
+			 * setBin Метод установки бинарных данных в словарь
+			 * @param buffer буфер с бинарными данными
+			 */
+			virtual void setBin(const vector <char> & buffer) const noexcept;
+			/**
+			 * getBin Метод извлечения данных arpa в бинарном виде
+			 * @param callback функция обратного вызова
+			 */
+			virtual void getBin(function <void (const vector <char> &, const u_short)> callback) const noexcept;
+		public:
+			/**
+			 * clear Метод очистки всех данных
 			 */
 			void clear();
-			/**
-			 * initPython Метод инициализации скриптов питона
-			 */
-			void initPython();
 			/**
 			 * clearBadwords Метод очистки списка плохих слов
 			 */
@@ -365,15 +342,6 @@ namespace anyks {
 			 * clearUserTokens Метод очистки пользовательских токенов
 			 */
 			void clearUserTokens() noexcept;
-		public:
-			/**
-			 * setAllTokenUnknown Метод установки всех токенов идентифицируемых как <unk>
-			 */
-			void setAllTokenUnknown() noexcept;
-			/**
-			 * setAllTokenDisable Метод установки всех токенов как не идентифицируемых
-			 */
-			void setAllTokenDisable() noexcept;
 		public:
 			/**
 			 * addBadword Метод добавления идентификатора похого слова в список
@@ -395,6 +363,20 @@ namespace anyks {
 			 * @param word слово для добавления
 			 */
 			void addGoodword(const string & word) noexcept;
+		public:
+			/**
+			 * setAllTokenUnknown Метод установки всех токенов идентифицируемых как <unk>
+			 */
+			void setAllTokenUnknown() noexcept;
+			/**
+			 * setAllTokenDisable Метод установки всех токенов как не идентифицируемых
+			 */
+			void setAllTokenDisable() noexcept;
+		public:
+			/**
+			 * initPython Метод инициализации скриптов питона
+			 */
+			void initPython();
 		public:
 			/**
 			 * setSize Метод установки размера n-граммы
@@ -458,11 +440,10 @@ namespace anyks {
 			 */
 			void setWordScript(const string & script) noexcept;
 			/**
-			 * setVocab Метод загрузки бинарных данных в словарь
-			 * @param buffer буфер с бинарными данными
-			 * @param arpa   нужно добавить только данные arpa
+			 * setWordPreprocessingMethod Метод установки функции препроцессинга слова
+			 * @param fn внешняя функция препроцессинга слова
 			 */
-			void setVocab(const vector <char> & buffer) noexcept;
+			void setWordPreprocessingMethod(wpres_t fn) noexcept;
 			/**
 			 * setTokenUnknown Метод установки списка токенов которых нужно идентифицировать как <unk>
 			 * @param options список токенов которых нужно идентифицировать как <unk>
@@ -473,11 +454,6 @@ namespace anyks {
 			 * @param options список не идентифицируемых токенов
 			 */
 			void setTokenDisable(const string & options) noexcept;
-			/**
-			 * setBin Метод установки бинарных данных в словарь
-			 * @param buffer буфер с бинарными данными
-			 */
-			void setBin(const vector <char> & buffer) const noexcept;
 			/**
 			 * setUserTokenScript Метод установки скрипта обработки пользовательских токенов
 			 * @param script скрипт python обработки пользовательских токенов
@@ -523,17 +499,44 @@ namespace anyks {
 			 * @param goodwords список хороших слов
 			 */
 			void setGoodwords(const vector <string> & goodwords) noexcept;
+			/**
+			 * setUserTokenMethod Метод установки функции обработки пользовательского токена
+			 * @param name слово - обозначение токена
+			 * @param fn   внешняя функция обрабатывающая пользовательский токен
+			 */
+			void setUserTokenMethod(const string & name, function <const bool (const string &, const string &)> fn) noexcept;
+		public:
+			/**
+			 * setVocab Метод загрузки бинарных данных в словарь
+			 * @param buffer буфер с бинарными данными
+			 * @param arpa   нужно добавить только данные arpa
+			 */
+			void setVocab(const vector <char> & buffer) noexcept;
+			/**
+			 * getVocab Метод извлечения данных словаря в бинарном виде
+			 * @param callback функция обратного вызова
+			 */
+			void getVocab(function <void (const vector <char> &, const u_short)> callback) const noexcept;
+		public:
 			/**
 			 * sentences Метод генерации предложений
 			 * @param callback функция обратного вызова
 			 */
-			void sentences(function <const bool (const wstring &)> callback) const noexcept;
+			virtual void sentences(function <const bool (const wstring &)> callback) const noexcept;
 			/**
 			 * getUppers Метод извлечения регистров для каждого слова
 			 * @param seq  последовательность слов для сборки контекста
 			 * @param upps список извлечённых последовательностей
 			 */
-			void getUppers(const vector <size_t> & seq, vector <size_t> & upps) const noexcept;
+			virtual void getUppers(const vector <size_t> & seq, vector <size_t> & upps) const noexcept;
+			/**
+			 * context Метод сборки текстового контекста из последовательности
+			 * @param seq  последовательность слов для сборки контекста
+			 * @param nwrd флаг разрешающий вывод системных токенов
+			 * @return     собранный текстовый контекст
+			 */
+			virtual const wstring context(const vector <size_t> & seq, const bool nwrd = false) const noexcept;
+		public:
 			/**
 			 * find Метод поиска n-грамм в тексте
 			 * @param text     текст в котором необходимо найти n-граммы
@@ -545,36 +548,14 @@ namespace anyks {
 			 * @param text     текст в котором необходимо найти n-граммы
 			 * @param callback функция обратного вызова
 			 */
-			void find(const wstring & text, function <void (const wstring &)> callback) const noexcept;
-			/**
-			 * getBin Метод извлечения данных arpa в бинарном виде
-			 * @param callback функция обратного вызова
-			 */
-			void getBin(function <void (const vector <char> &, const u_short)> callback) const noexcept;
-			/**
-			 * getVocab Метод извлечения данных словаря в бинарном виде
-			 * @param callback функция обратного вызова
-			 */
-			void getVocab(function <void (const vector <char> &, const u_short)> callback) const noexcept;
+			virtual void find(const wstring & text, function <void (const wstring &)> callback) const noexcept;
+		public:
 			/**
 			 * read Метод чтения данных из файла arpa
 			 * @param filename адрес файла для чтения
 			 * @param status   функция вывода статуса
 			 */
 			void read(const string & filename, function <void (const u_short)> status = nullptr) noexcept;
-		public:
-			/**
-			 * setWordPreprocessingMethod Метод установки функции препроцессинга слова
-			 * @param fn внешняя функция препроцессинга слова
-			 */
-			void setWordPreprocessingMethod(wpres_t fn) noexcept;
-			/**
-			 * setUserTokenMethod Метод установки функции обработки пользовательского токена
-			 * @param name слово - обозначение токена
-			 * @param fn   внешняя функция обрабатывающая пользовательский токен
-			 */
-			void setUserTokenMethod(const string & name, function <const bool (const string &, const string &)> fn) noexcept;
-		public:
 			/**
 			 * sentencesToFile Метод сборки указанного количества предложений и записи в файл
 			 * @param counts   количество предложений для сборки
@@ -627,6 +608,20 @@ namespace anyks {
 			 * @return установленные опции
 			 */
 			const u_int getOptions() const noexcept;
+			/**
+			 * getUserTokenId Метод получения идентификатора пользовательского токена
+			 * @param name слово для которого нужно получить идентификатор
+			 * @return     идентификатор пользовательского токена соответствующий слову
+			 */
+			const size_t getUserTokenId(const string & name) const noexcept;
+			/**
+			 * getIdw Метод генерирования идентификатора слова
+			 * @param  word  слово для генерации
+			 * @param  check нужно выполнить дополнительную проверку слова
+			 * @return       идентификатор слова
+			 */
+			const size_t getIdw(const wstring & word, const bool check = true) const noexcept;
+		public:
 			/**
 			 * getUnknown Метод извлечения неизвестного слова
 			 * @return установленное неизвестное слово
@@ -642,6 +637,21 @@ namespace anyks {
 			 * @return адрес скрипта python обработки пользовательских токенов
 			 */
 			const string & getUserTokenScript() const noexcept;
+			/**
+			 * getUserTokenWord Метод получения пользовательского токена по его идентификатору
+			 * @param idw идентификатор пользовательского токена
+			 * @return    пользовательский токен соответствующий идентификатору
+			 */
+			const string getUserTokenWord(const size_t idw) const noexcept;
+		public:
+			/**
+			 * word Метод извлечения слова по его идентификатору
+			 * @param idw идентификатор слова
+			 * @param ups регистры слова
+			 * @return    слово соответствующее идентификатору
+			 */
+			const word_t word(const size_t idw, const size_t ups = 0) const noexcept;
+		public:
 			/**
 			 * getBadwords Метод извлечения чёрного списка
 			 * @return чёрный список слов
@@ -667,6 +677,7 @@ namespace anyks {
 			 * @return список пользовательских токенов
 			 */
 			const vector <string> & getUserTokens() const noexcept;
+		public:
 			/**
 			 * grams Метод проверки количества найденных в тексте n-грамм
 			 * @param text текст для расчёта
@@ -679,6 +690,7 @@ namespace anyks {
 			 * @return    количество найденных n-грамм
 			 */
 			const size_t grams(const vector <size_t> & seq) const noexcept;
+		public:
 			/**
 			 * bigrams Метод проверки количества найденных в тексте биграмм
 			 * @param text текст для расчёта
@@ -691,6 +703,7 @@ namespace anyks {
 			 * @return    количество найденных биграмм
 			 */
 			const size_t bigrams(const vector <size_t> & seq) const noexcept;
+		public:
 			/**
 			 * trigrams Метод проверки количества найденных в тексте триграмм
 			 * @param text текст для расчёта
@@ -703,18 +716,6 @@ namespace anyks {
 			 * @return    количество найденных триграмм
 			 */
 			const size_t trigrams(const vector <size_t> & seq) const noexcept;
-			/**
-			 * getUserTokenWord Метод получения пользовательского токена по его идентификатору
-			 * @param idw идентификатор пользовательского токена
-			 * @return    пользовательский токен соответствующий идентификатору
-			 */
-			const string getUserTokenWord(const size_t idw) const noexcept;
-			/**
-			 * getUserTokenId Метод получения идентификатора пользовательского токена
-			 * @param name слово для которого нужно получить идентификатор
-			 * @return     идентификатор пользовательского токена соответствующий слову
-			 */
-			const size_t getUserTokenId(const string & name) const noexcept;
 		public:
 			/**
 			 * Alm Конструктор
@@ -739,8 +740,307 @@ namespace anyks {
 			/**
 			 * ~Alm Деструктор
 			 */
-			~Alm() noexcept;
+			virtual ~Alm() noexcept;
 	} alm_t;
+	/**
+	 * Alm1 Класс alm первого типа
+	 */
+	typedef class Alm1 : public Alm {
+		private:
+			/**
+			 * Структура словаря языковой модели
+			 */
+			typedef struct Arpa : std::map <size_t, Arpa> {
+				/**
+				 * Регистры слова
+				 */
+				size_t uppers;
+				/**
+				 * Частота последовательности и
+				 * обратная частота последовательности
+				 */
+				double weight, backoff;
+				/**
+				 * Arpa Конструктор
+				 */
+				Arpa() : uppers(0), weight(log(0)), backoff(0.0) {}
+			} arpa_t;
+			// Словарь языковой модели
+			mutable arpa_t arpa;
+		private:
+			/**
+			 * exist Метод проверки существования последовательности
+			 * @param seq список слов последовательности
+			 * @return    результат проверки
+			 */
+			virtual const bool exist(const vector <size_t> & seq) const noexcept;
+			/**
+			 * backoff Метод извлечения обратной частоты последовательности
+			 * @param seq последовательность для извлечения обратной частоты
+			 * @return    обратная частота последовательности
+			 */
+			virtual const double backoff(const vector <size_t> & seq) const noexcept;
+			/**
+			 * weight Метод извлечения веса последовательности
+			 * @param seq последовательность для извлечения веса
+			 * @return    вес последовательноси и n-грамма для которой она получена
+			 */
+			virtual const pair <u_short, double> weight(const vector <size_t> & seq) const noexcept;
+			/**
+			 * frequency Метод извлечения частоты n-граммы
+			 * @param seq список слов последовательности
+			 * @return    частота и обратная частота n-граммы
+			 */
+			virtual const pair <double, double> frequency(const vector <size_t> & seq) const noexcept;
+		private:
+			/**
+			 * set Метод установки последовательности в словарь
+			 * @param seq последовательность слов для установки
+			 */
+			virtual void set(const vector <alm_t::seq_t> & seq) const noexcept;
+			/**
+			 * set Метод установки последовательности в словарь
+			 * @param seq     список идентификаторов слов которые нужно добавить
+			 * @param uppers  список верхних регистров последнего слова последовательности
+			 * @param weight  вес n-граммы из файла arpa
+			 * @param backoff обратная частота документа из файла arpa
+			 */
+			virtual void set(const vector <size_t> & seq, const size_t uppers, const double weight, const double backoff) const noexcept;
+		public:
+			/**
+			 * clear Метод очистки всех данных
+			 */
+			virtual void clear();
+		public:
+			/**
+			 * perplexity Метод расчёта перплексии
+			 * @param  seq список последовательностей
+			 * @return     результат расчёта
+			 */
+			virtual const ppl_t perplexity(const vector <size_t> & seq) const noexcept;
+			/**
+			 * check Метод проверки существования последовательности
+			 * @param seq      список слов последовательности
+			 * @param accurate режим точной проверки
+			 * @return         результат проверки
+			 */
+			virtual const pair <bool, size_t> check(const vector <size_t> & seq, const bool accurate = false) const noexcept;
+		public:
+			/**
+			 * setBin Метод установки бинарных данных в словарь
+			 * @param buffer буфер с бинарными данными
+			 */
+			virtual void setBin(const vector <char> & buffer) const noexcept;
+			/**
+			 * getBin Метод извлечения данных arpa в бинарном виде
+			 * @param callback функция обратного вызова
+			 */
+			virtual void getBin(function <void (const vector <char> &, const u_short)> callback) const noexcept;
+		public:
+			/**
+			 * sentences Метод генерации предложений
+			 * @param callback функция обратного вызова
+			 */
+			virtual void sentences(function <const bool (const wstring &)> callback) const noexcept;
+			/**
+			 * getUppers Метод извлечения регистров для каждого слова
+			 * @param seq  последовательность слов для сборки контекста
+			 * @param upps список извлечённых последовательностей
+			 */
+			virtual void getUppers(const vector <size_t> & seq, vector <size_t> & upps) const noexcept;
+			/**
+			 * find Метод поиска n-грамм в тексте
+			 * @param text     текст в котором необходимо найти n-граммы
+			 * @param callback функция обратного вызова
+			 */
+			virtual void find(const wstring & text, function <void (const wstring &)> callback) const noexcept;
+		public:
+			/**
+			 * context Метод сборки текстового контекста из последовательности
+			 * @param seq  последовательность слов для сборки контекста
+			 * @param nwrd флаг разрешающий вывод системных токенов
+			 * @return     собранный текстовый контекст
+			 */
+			virtual const wstring context(const vector <size_t> & seq, const bool nwrd = false) const noexcept;
+		public:
+			/**
+			 * Alm1 Конструктор
+			 */
+			Alm1() noexcept : alm_t() {};
+			/**
+			 * Alm1 Конструктор
+			 * @param alphabet объект алфавита
+			 */
+			Alm1(const alphabet_t * alphabet) noexcept : alm_t(alphabet) {};
+			/**
+			 * Alm1 Конструктор
+			 * @param tokenizer объект токенизатора
+			 */
+			Alm1(const tokenizer_t * tokenizer) noexcept : alm_t(tokenizer) {};
+			/**
+			 * Alm1 Конструктор
+			 * @param alphabet  объект алфавита
+			 * @param tokenizer объект токенизатора
+			 */
+			Alm1(const alphabet_t * alphabet, const tokenizer_t * tokenizer) noexcept : alm_t(alphabet, tokenizer) {};
+			/**
+			 * ~Alm1 Деструктор
+			 */
+			virtual ~Alm1() noexcept;
+	} alm1_t;
+	/**
+	 * Alm2 Класс alm второго типа
+	 */
+	typedef class Alm2 : public Alm1 {
+		private:
+			/**
+			 * Ngram Структура N-граммы
+			 */
+			typedef struct Ngram {
+				/**
+				 * Регистры слова
+				 */
+				size_t uppers;
+				/**
+				 * Частота последовательности и
+				 * обратная частота последовательности
+				 */
+				double weight, backoff;
+				/**
+				 * Ngram Конструктор
+				 */
+				Ngram() : uppers(0), weight(log(0)), backoff(0.0) {}
+			} ngram_t;
+			// Создаём тип данных структуры Лес, для данных arpa
+			typedef std::map <u_short, std::map <size_t, ngram_t>> arpa_t;
+			// Словарь языковой модели
+			mutable arpa_t arpa;
+		private:
+			/**
+			 * exist Метод проверки существования последовательности
+			 * @param seq список слов последовательности
+			 * @return    результат проверки
+			 */
+			const bool exist(const vector <size_t> & seq) const noexcept;
+			/**
+			 * backoff Метод извлечения обратной частоты последовательности
+			 * @param seq последовательность для извлечения обратной частоты
+			 * @return    обратная частота последовательности
+			 */
+			const double backoff(const vector <size_t> & seq) const noexcept;
+			/**
+			 * weight Метод извлечения веса последовательности
+			 * @param seq последовательность для извлечения веса
+			 * @return    вес последовательноси и n-грамма для которой она получена
+			 */
+			const pair <u_short, double> weight(const vector <size_t> & seq) const noexcept;
+			/**
+			 * frequency Метод извлечения частоты n-граммы
+			 * @param seq список слов последовательности
+			 * @return    частота и обратная частота n-граммы
+			 */
+			const pair <double, double> frequency(const vector <size_t> & seq) const noexcept;
+		private:
+			/**
+			 * set Метод установки последовательности в словарь
+			 * @param seq последовательность слов для установки
+			 */
+			void set(const vector <alm_t::seq_t> & seq) const noexcept;
+			/**
+			 * set Метод установки последовательности в словарь
+			 * @param seq     список идентификаторов слов которые нужно добавить
+			 * @param uppers  список верхних регистров последнего слова последовательности
+			 * @param weight  вес n-граммы из файла arpa
+			 * @param backoff обратная частота документа из файла arpa
+			 */
+			void set(const vector <size_t> & seq, const size_t uppers, const double weight, const double backoff) const noexcept;
+		public:
+			/**
+			 * clear Метод очистки всех данных
+			 */
+			void clear();
+		public:
+			/**
+			 * perplexity Метод расчёта перплексии
+			 * @param  seq список последовательностей
+			 * @return     результат расчёта
+			 */
+			const ppl_t perplexity(const vector <size_t> & seq) const noexcept;
+			/**
+			 * check Метод проверки существования последовательности
+			 * @param seq      список слов последовательности
+			 * @param accurate режим точной проверки
+			 * @return         результат проверки
+			 */
+			const pair <bool, size_t> check(const vector <size_t> & seq, const bool accurate = false) const noexcept;
+		public:
+			/**
+			 * setBin Метод установки бинарных данных в словарь
+			 * @param buffer буфер с бинарными данными
+			 */
+			void setBin(const vector <char> & buffer) const noexcept;
+			/**
+			 * setBin2 Метод установки бинарных данных в словарь
+			 * @param buffer буфер с бинарными данными
+			 */
+			void setBin2(const vector <char> & buffer) const noexcept;
+			/**
+			 * getBin Метод извлечения данных arpa в бинарном виде
+			 * @param callback функция обратного вызова
+			 */
+			void getBin(function <void (const vector <char> &, const u_short)> callback) const noexcept;
+		public:
+			/**
+			 * sentences Метод генерации предложений
+			 * @param callback функция обратного вызова
+			 */
+			void sentences(function <const bool (const wstring &)> callback) const noexcept;
+			/**
+			 * getUppers Метод извлечения регистров для каждого слова
+			 * @param seq  последовательность слов для сборки контекста
+			 * @param upps список извлечённых последовательностей
+			 */
+			void getUppers(const vector <size_t> & seq, vector <size_t> & upps) const noexcept;
+			/**
+			 * find Метод поиска n-грамм в тексте
+			 * @param text     текст в котором необходимо найти n-граммы
+			 * @param callback функция обратного вызова
+			 */
+			void find(const wstring & text, function <void (const wstring &)> callback) const noexcept;
+		public:
+			/**
+			 * context Метод сборки текстового контекста из последовательности
+			 * @param seq  последовательность слов для сборки контекста
+			 * @param nwrd флаг разрешающий вывод системных токенов
+			 * @return     собранный текстовый контекст
+			 */
+			const wstring context(const vector <size_t> & seq, const bool nwrd = false) const noexcept;
+		public:
+			/**
+			 * Alm2 Конструктор
+			 */
+			Alm2() noexcept : Alm1() {};
+			/**
+			 * Alm2 Конструктор
+			 * @param alphabet объект алфавита
+			 */
+			Alm2(const alphabet_t * alphabet) noexcept : Alm1(alphabet) {};
+			/**
+			 * Alm2 Конструктор
+			 * @param tokenizer объект токенизатора
+			 */
+			Alm2(const tokenizer_t * tokenizer) noexcept : Alm1(tokenizer) {};
+			/**
+			 * Alm2 Конструктор
+			 * @param alphabet  объект алфавита
+			 * @param tokenizer объект токенизатора
+			 */
+			Alm2(const alphabet_t * alphabet, const tokenizer_t * tokenizer) noexcept : Alm1(alphabet, tokenizer) {};
+			/**
+			 * ~Alm2 Деструктор
+			 */
+			~Alm2() noexcept;
+	} alm2_t;
 };
 
 #endif // _ANYKS_LANGUAGE_MODEL_
