@@ -23,7 +23,8 @@
 #include <chrono>
 #include <cstdarg>
 #include <fstream>
-#include <string.h>
+#include <cstring>
+#include <algorithm>
 #include <sys/types.h>
 /**
  * Наши модули
@@ -60,6 +61,8 @@ namespace anyks {
 		private:
 			// Латинский тип алфавита
 			bool typeLatian = true;
+			// Разрешить апостроф как часть слова
+			bool apostrophe = false;
 			// Алфавит словаря
 			wstring alphabet = L"abcdefghijklmnopqrstuvwxyz";
 		private:
@@ -138,10 +141,15 @@ namespace anyks {
 			 * @return алфавит языка
 			 */
 			const string get() const noexcept;
+			/**
+			 * get Метод получения алфавита языка
+			 * @return алфавит языка
+			 */
+			const wstring & wget() const noexcept;
 			/**
 			 * trim Метод удаления пробелов вначале и конце текста
 			 * @param  text текст для удаления пробелов
-			 * @return      текст без пробелов
+			 * @return      результат работы функции
 			 */
 			const string trim(const string & text) const noexcept;
 			/**
@@ -210,7 +218,7 @@ namespace anyks {
 			/**
 			 * trim Метод удаления пробелов вначале и конце текста
 			 * @param  text текст для удаления пробелов
-			 * @return      текст без пробелов
+			 * @return      результат работы функции
 			 */
 			const wstring trim(const wstring & text) const noexcept;
 			/**
@@ -314,6 +322,11 @@ namespace anyks {
 			 * @return результат проверки
 			 */
 			const bool altemp() const noexcept;
+			/**
+			 * isAllowApostrophe Метод проверки разрешения апострофа
+			 * @return результат проверки
+			 */
+			const bool isAllowApostrophe() const noexcept;
 			/**
 			 * isUrl Метод проверки соответствия слова url адресу
 			 * @param word слово для проверки
@@ -462,6 +475,10 @@ namespace anyks {
 			 * clear Метод очистки собранных данных
 			 */
 			void clear() noexcept;
+			/**
+			 * switchAllowApostrophe Метод разрешения или запрещения апострофа как части слова
+			 */
+			void switchAllowApostrophe() noexcept;
 			/**
 			 * log Метод вывода текстовой информации в консоль или файл
 			 * @param format   формат строки вывода
