@@ -8,51 +8,44 @@
 
 #include <alphabet.hpp>
 
+
 // Устанавливаем шаблон функции
-template <typename C, typename T, typename A>
-/**
- * strim Функция удаления начальных и конечных пробелов
- * @param str строка для обработки
- * @param loc локаль
- * @return    результат работы функции
- */
-basic_string <C, T, A> strim(const basic_string <C, T, A> & str, const locale & loc = locale::classic()) noexcept {
-	// Запоминаем итератор на первый левый символ
-	auto begin = str.begin();
-	// Переходим по всем символам в слове и проверяем является ли символ - символом пробела, если нашли то смещаем итератор
-	while((begin != str.end()) && isspace(* begin, loc)) ++begin;
-	// Если прошли все слово целиком значит пробелов нет и мы выходим
-	if(begin == str.end()) return {};
-	// Запоминаем итератор на первый правый символ
-	auto rbegin = str.rbegin();
-	// Переходим по всем символам в слове и проверяем является ли символ - символом пробела, если нашли то смещаем итератор
-	while(rbegin != str.rend() && isspace(* rbegin, loc)) ++rbegin;
-	// Выводим результат
-	return {begin, rbegin.base()};
-}
-// Устанавливаем шаблон функции
-template <typename A, typename B>
+template <typename T>
 /**
  * split Метод разделения строк на составляющие
  * @param str   строка для поиска
  * @param delim разделитель
  * @param v     результирующий вектор
  */
-void split(const A & str, const A & delim, B & v) noexcept {
+void split(const wstring & str, const wstring & delim, T & v) noexcept {
+	/**
+	 * trimFn Метод удаления пробелов вначале и конце текста
+	 * @param text текст для удаления пробелов
+	 * @return     результат работы функции
+	 */
+	function <const wstring (const wstring &)> trimFn = [](const wstring & text) noexcept {
+		// Получаем временный текст
+		wstring tmp = text;
+		// Выполняем удаление пробелов по краям
+		tmp.erase(tmp.begin(), find_if_not(tmp.begin(), tmp.end(), [](wchar_t c){ return iswspace(c); }));
+		tmp.erase(find_if_not(tmp.rbegin(), tmp.rend(), [](wchar_t c){ return iswspace(c); }).base(), tmp.end());
+		// Выводим результат
+		return tmp;
+	};
 	// Очищаем словарь
 	v.clear();
 	// Получаем счётчики перебора
 	size_t i = 0, j = str.find(delim);
 	const size_t len = delim.length();
 	// Выполняем разбиение строк
-	while(j != A::npos){
-		v.push_back(strim(str.substr(i, j - i)));
+	while(j != wstring::npos){
+		v.push_back(trimFn(str.substr(i, j - i)));
 		i = ++j + (len - 1);
 		j = str.find(delim, j);
-		if(j == A::npos) v.push_back(strim(str.substr(i, str.length())));
+		if(j == wstring::npos) v.push_back(trimFn(str.substr(i, str.length())));
 	}
 	// Если слово передано а вектор пустой, тогда создаем вектори из 1-го элемента
-	if(!str.empty() && v.empty()) v.push_back(strim(str));
+	if(!str.empty() && v.empty()) v.push_back(trimFn(str));
 }
 /**
  * cbegin Метод итератор начала списка
@@ -102,14 +95,27 @@ const string anyks::Alphabet::get() const noexcept {
 	// Выводим результат
 	return this->convert(this->alphabet);
 }
+/**
+ * get Метод получения алфавита языка
+ * @return алфавит языка
+ */
+const wstring & anyks::Alphabet::wget() const noexcept {
+	// Выводим результат
+	return this->alphabet;
+}
 /**
  * trim Метод удаления пробелов вначале и конце текста
- * @param  text текст для удаления пробелов
- * @return      текст без пробелов
+ * @param text текст для удаления пробелов
+ * @return     результат работы функции
  */
 const string anyks::Alphabet::trim(const string & text) const noexcept {
+	// Получаем временный текст
+	string tmp = text;
+	// Выполняем удаление пробелов по краям
+	tmp.erase(tmp.begin(), find_if_not(tmp.begin(), tmp.end(), [](char c){ return isspace(c); }));
+	tmp.erase(find_if_not(tmp.rbegin(), tmp.rend(), [](char c){ return isspace(c); }).base(), tmp.end());
 	// Выводим результат
-	return (!text.empty() ? strim(text) : "");
+	return tmp;
 }
 /**
  * toLower Метод перевода русских букв в нижний регистр
@@ -316,12 +322,17 @@ const wchar_t anyks::Alphabet::toUpper(const wchar_t letter) const noexcept {
 }
 /**
  * trim Метод удаления пробелов вначале и конце текста
- * @param  text текст для удаления пробелов
- * @return      текст без пробелов
+ * @param text текст для удаления пробелов
+ * @return     результат работы функции
  */
 const wstring anyks::Alphabet::trim(const wstring & text) const noexcept {
+	// Получаем временный текст
+	wstring tmp = text;
+	// Выполняем удаление пробелов по краям
+	tmp.erase(tmp.begin(), find_if_not(tmp.begin(), tmp.end(), [this](wchar_t c){ return this->isSpace(c); }));
+	tmp.erase(find_if_not(tmp.rbegin(), tmp.rend(), [this](wchar_t c){ return this->isSpace(c); }).base(), tmp.end());
 	// Выводим результат
-	return (!text.empty() ? strim(text) : L"");
+	return tmp;
 }
 /**
  * convert Метод конвертирования строки в строку utf-8
@@ -505,7 +516,7 @@ const wstring anyks::Alphabet::delBrokenInWord(const wstring & word) const noexc
 			return (
 				(this->allowedSymbols.count(letter) < 1) &&
 				(letter != L'\r') && (letter != L'\n') &&
-				!iswspace(letter) && !this->check(letter) && !this->isNumber({letter})
+				!iswspace(letter) && !this->check(letter)
 			);
 		}), result.end());
 	}
@@ -741,6 +752,14 @@ const bool anyks::Alphabet::altemp() const noexcept {
 	// Выводим результат проверки
 	return this->alters.empty();
 }
+/**
+ * isAllowApostrophe Метод проверки разрешения апострофа
+ * @return результат проверки
+ */
+const bool anyks::Alphabet::isAllowApostrophe() const noexcept {
+	// Выводим результат проверки апострофа
+	return this->apostrophe;
+}
 /**
  * isUrl Метод проверки соответствия слова url адресу
  * @param word слово для проверки
@@ -799,7 +818,7 @@ const bool anyks::Alphabet::isAbbr(const wstring & word) const noexcept {
 	// Результат работы функции
 	bool result = false;
 	// Если слово передано
-	if(!word.empty() && (word.length() > 1) && (word.back() != L'-') && this->check(word.front())){
+	if(!word.empty() && (word.length() > 1) && (word.front() != L'-') && (word.back() != L'-')){
 		// Позиция пробела
 		size_t pos = wstring::npos;
 		// Ищем дефис в слове
@@ -829,57 +848,62 @@ const bool anyks::Alphabet::isAbbr(const wstring & word) const noexcept {
 								// Получаем текущую букву
 								letter = tmp.at(i);
 								// Проверяем соответствует ли буква
-								result = this->check(letter);
+								result = (this->letters.count(letter) > 0);
 								// Проверяем вторую букву в слове
 								if(result && (i != j)){
 									// Получаем текущую букву
 									letter = tmp.at(j);
 									// Проверяем соответствует ли буква
-									result = this->check(letter);
+									result = (this->letters.count(letter) > 0);
 								}
 								// Если буква не соответствует, выходим
 								if(!result) break;
 							}
 						// Если строка всего из одного символа
-						} else result = this->check(tmp.front());
+						} else result = (this->letters.count(tmp.front()) > 0);
 					}
 					// Выводим результат
 					return result;
 				};
 				// Получаем суффикс
-				const wstring & suffix = word.substr(pos + 1);
+				const wstring & suffix = this->toLower(word.substr(pos + 1));
 				// Если суффикс не является числом
 				result = (!this->isNumber(suffix) && isAllowedFn(suffix));
 			}
 		// Если это не дефис
 		} else {
-			// Флаг найденой точки
-			bool point = false;
-			// Длина слова до точки
-			u_short length = 1;
-			// Текущая буква в слове
-			wchar_t letter = 0;
-			// Выполняем переход по всему слову
-			for(size_t i = 1; i < word.length(); i++){
-				// Получаем текущую букву слова
-				letter = word.at(i);
-				// Если это не точка
-				if((result = (letter == L'.'))){
-					// Сбрасываем длину слова
-					length = 0;
-					// Запоминаем что точка найдена
-					point = true;
-				// Если слово не прошло проверку
-				} else if((length > 3) || !this->check(letter)) {
-					// Запоминаем, что результат не получен
-					result = false;
-					// Выходим из цикла
-					break;
-				// Увеличиваем количество обработанных букв
-				} else if((result = true)) length++;
+			// Переводим слово в нижний регистр
+			const wstring & tmp = this->toLower(word);
+			// Если первый символ является разрешённым
+			if(this->letters.count(tmp.front()) > 0){
+				// Флаг найденой точки
+				bool point = false;
+				// Длина слова до точки
+				u_short length = 1;
+				// Текущая буква в слове
+				wchar_t letter = 0;
+				// Выполняем переход по всему слову
+				for(size_t i = 1; i < tmp.length(); i++){
+					// Получаем текущую букву слова
+					letter = tmp.at(i);
+					// Если это не точка
+					if((result = (letter == L'.'))){
+						// Сбрасываем длину слова
+						length = 0;
+						// Запоминаем что точка найдена
+						point = true;
+					// Если слово не прошло проверку
+					} else if((length > 3) || (this->letters.count(letter) < 1)) {
+						// Запоминаем, что результат не получен
+						result = false;
+						// Выходим из цикла
+						break;
+					// Увеличиваем количество обработанных букв
+					} else if((result = true)) length++;
+				}
+				// Если точка не найдена, сбрасываем результат
+				if(!point) result = false;
 			}
-			// Если точка не найдена, сбрасываем результат
-			if(!point) result = false;
 		}
 	}
 	// Выводим результат
@@ -913,26 +937,53 @@ const bool anyks::Alphabet::isLatian(const wstring & str) const noexcept {
 	if(!str.empty()){
 		// Длина слова
 		const size_t length = str.length();
+		// Переводим слово в нижний регистр
+		const wstring & tmp = this->toLower(str);
 		// Если длина слова больше 1-го символа
 		if(length > 1){
 			/**
-			 * checkFn Метод проверки соответствия буквы
-			 * @param letter буква для проверки
-			 * @return       результат проверки
+			 * checkFn Функция проверки на валидность символа
+			 * @param text  текст для проверки
+			 * @param index индекс буквы в слове
+			 * @return      результат проверки
 			 */
-			auto checkFn = [this](const wchar_t letter) noexcept {
-				// Результат проверки
-				return ((letter == L'-') || (letter == L'\'') || (this->latian.count(letter) > 0));
+			auto checkFn = [this](const wstring & text, const size_t index) noexcept {
+				// Результат работы функции
+				bool result = false;
+				// Получаем текущую букву
+				const wchar_t letter = text.at(index);
+				// Если буква не первая и не последняя
+				if((index > 0) && (index < (text.length() - 1))){
+					// Получаем предыдущую букву
+					const wchar_t first = text.at(index - 1);
+					// Получаем следующую букву
+					const wchar_t second = text.at(index + 1);
+					// Если это дефис
+					result = ((letter == L'-') && (first != L'-') && (second != L'-'));
+					// Если проверка не пройдена, проверяем на апостроф
+					if(!result){
+						// Выполняем проверку на апостроф
+						result = (
+							(letter == L'\'') && ((this->isAllowApostrophe() && ((first != L'\'') && (second != L'\''))) ||
+							((this->latian.count(first) > 0) && (this->latian.count(second) > 0)))
+						);
+					}
+					// Если результат не получен
+					if(!result) result = (this->latian.count(letter) > 0);
+				// Выводим проверку как она есть
+				} else result = (this->latian.count(letter) > 0);
+				// Выводим результат
+				return result;
 			};
 			// Переходим по всем буквам слова
 			for(size_t i = 0, j = (length - 1); j > ((length / 2) - 1); i++, j--){
 				// Проверяем является ли слово латинским
-				result = (i == j ? checkFn(str.at(i)) : checkFn(str.at(i)) && checkFn(str.at(j)));
+				result = (i == j ? checkFn(tmp, i) : checkFn(tmp, i) && checkFn(tmp, j));
 				// Если слово не соответствует тогда выходим
 				if(!result) break;
 			}
 		// Если символ всего один, проверяем его так
-		} else result = (this->latian.count(str.front()) > 0);
+		} else result = (this->latian.count(tmp.front()) > 0);
 	}
 	// Выводим результат
 	return result;
@@ -1076,26 +1127,49 @@ const bool anyks::Alphabet::isAllowed(const wstring & word) const noexcept {
 		const wstring & tmp = this->toLower(word);
 		// Если строка длиннее 1-го символа
 		if(length > 1){
-			// Текущая буква
-			wchar_t letter = 0;
+			/**
+			 * checkFn Функция проверки на валидность символа
+			 * @param text  текст для проверки
+			 * @param index индекс буквы в слове
+			 * @return      результат проверки
+			 */
+			auto checkFn = [this](const wstring & text, const size_t index) noexcept {
+				// Результат работы функции
+				bool result = false;
+				// Получаем текущую букву
+				const wchar_t letter = text.at(index);
+				// Если буква не первая и не последняя
+				if((index > 0) && (index < (text.length() - 1))){
+					// Получаем предыдущую букву
+					const wchar_t first = text.at(index - 1);
+					// Получаем следующую букву
+					const wchar_t second = text.at(index + 1);
+					// Если это дефис
+					result = ((letter == L'-') && (first != L'-') && (second != L'-'));
+					// Если проверка не пройдена, проверяем на апостроф
+					if(!result){
+						// Выполняем проверку на апостроф
+						result = (
+							(letter == L'\'') && ((this->isAllowApostrophe() && ((first != L'\'') && (second != L'\''))) ||
+							((this->latian.count(first) > 0) && (this->latian.count(second) > 0)))
+						);
+					}
+					// Если результат не получен
+					if(!result) result = this->check(letter);
+				// Выводим проверку как она есть
+				} else result = this->check(letter);
+				// Выводим результат
+				return result;
+			};
 			// Выполняем переход по всем буквам слова
 			for(size_t i = 0, j = (length - 1); j > ((length / 2) - 1); i++, j--){
-				// Получаем текущую букву
-				letter = tmp.at(i);
-				// Проверяем соответствует ли буква
-				result = ((letter == L'-') || this->check(letter));
-				// Проверяем вторую букву в слове
-				if(result && (i != j)){
-					// Получаем текущую букву
-					letter = tmp.at(j);
-					// Проверяем соответствует ли буква
-					result = ((letter == L'-') || this->check(letter));
-				}
-				// Если буква не соответствует, выходим
+				// Проверяем является ли слово разрешённым
+				result = (i == j ? checkFn(tmp, i) : checkFn(tmp, i) && checkFn(tmp, j));
+				// Если слово не соответствует тогда выходим
 				if(!result) break;
 			}
 		// Если строка всего из одного символа
-		} else result = ((tmp.front() == L'-') || this->check(tmp.front()));
+		} else result = this->check(tmp.front());
 	}
 	// Выводим результат
 	return result;
@@ -1151,7 +1225,7 @@ const bool anyks::Alphabet::rest(wstring & word) const noexcept {
 			// Если это латинский алфавит
 			if(this->checkLatian({letter})) latian.emplace(letter, pos);
 			// Если это нормальная буква
-			else if(!this->typeLatian && this->check(letter)) normal.emplace(letter, pos);
+			else if(!this->typeLatian && (this->letters.count(letter) > 0)) normal.emplace(letter, pos);
 			// Иначе это другие символы
 			else other.emplace(letter, pos);
 		};
@@ -1332,7 +1406,7 @@ const bool anyks::Alphabet::checkSimilars(const wstring & str) const noexcept {
 						(letter != L'-') &&
 						!this->isNumber({letter}) &&
 						!this->isLatian({letter}) &&
-						this->check(letter)
+						(this->letters.count(letter) > 0)
 					);
 				}
 				// Выводим результат
@@ -1494,6 +1568,13 @@ void anyks::Alphabet::clear() noexcept {
 	// Формируем алфавит
 	this->set(this->convert(alphabet));
 }
+/**
+ * switchAllowApostrophe Метод разрешения или запрещения апострофа как части слова
+ */
+void anyks::Alphabet::switchAllowApostrophe() noexcept {
+	// Выполняем переключение разрешения использования апострофа
+	this->apostrophe = !this->apostrophe;
+}
 /**
  * log Метод вывода текстовой информации в консоль или файл
  * @param format   формат строки вывода
